# -*- coding: utf-8 -*-
"""
Created on Wed May 16 14:27:21 2018
@author: s161488
"""
import numpy as np
import tensorflow as tf


def Prepare_Active_Data(path, num_tr, combine=True):
    """
    choose_index_tr: worst 16+best 16
    or middle 32
    this num_tr should be 1/2*total_number_of_training_images_at_inital_step
    I have tried it for 32, then I am going to check 16
    
    """
    val_num_im = 96
    tot_numeric_index = np.arange(900)
    if combine is True:
        tr_select_numeric_index = np.concatenate([tot_numeric_index[:num_tr], tot_numeric_index[-num_tr:]], axis=0)
    else:
        tr_select_numeric_index = tot_numeric_index[340:(340 + num_tr * 2)]
    val_select_numeric_index = tot_numeric_index[500:(500 + val_num_im)]
    pool_numeric_index = np.delete(tot_numeric_index,
                                   np.concatenate([tr_select_numeric_index, val_select_numeric_index], axis=0))
    im_seg_score = np.load('/home/s161488/Exp_Stat/Skin_Lesion/init_segment_score.npy')
    sorted_index = np.argsort(im_seg_score)

    data_set = np.load(path, encoding='latin1').item()
    images = np.array(data_set['image'])
    labels = np.array(data_set['label'])
    edges = np.array(data_set['edge'])

    labels = np.expand_dims(labels, axis=-1)
    edges = np.expand_dims(edges, axis=-1)
    tr_select_image_index = np.sort(sorted_index[tr_select_numeric_index])
    val_select_image_index = np.sort(sorted_index[val_select_numeric_index])
    pl_select_image_index = np.sort(sorted_index[pool_numeric_index])

    X_image_tr, Y_label_tr, Y_edge_tr = Extract_Diff_Data(images, labels, edges, tr_select_image_index)
    X_image_pl, Y_label_pl, Y_edge_pl = Extract_Diff_Data(images, labels, edges, pl_select_image_index)
    X_image_val, Y_label_val, Y_edge_val = Extract_Diff_Data(images, labels, edges, val_select_image_index)
    print("-------------There are %d training images %d validation images %d pool images" % (
        np.shape(X_image_tr)[0], np.shape(X_image_val)[0], np.shape(X_image_pl)[0]))
    Data_Train = [X_image_tr, Y_label_tr, Y_edge_tr]
    Data_Pool = [X_image_pl, Y_label_pl, Y_edge_pl]
    Data_Val = [X_image_val, Y_label_val, Y_edge_val]
    return Data_Train, Data_Pool, Data_Val


def Generate_Batch(X_image_tr, Y_label_tr, Y_edge_tr, Y_binary_mask_tr, batch_index, batch_size):
    """The data augmentation include: rotation, random_brightness, crop_and_pad
    
    Args:
        X_image_tr, Y_label_tr, and Y_edge_tr are already shuffled
        
    
    Return:
        [X_image_aug, Y_label_aug, Y_edge_aug]
    """
    X_image_batch = X_image_tr[batch_index:(batch_size + batch_index), :, :, :]
    Y_label_batch = Y_label_tr[batch_index:(batch_size + batch_index), :, :, :]
    Y_edge_batch = Y_edge_tr[batch_index:(batch_size + batch_index), :, :, :]
    Y_binary_mask_batch = Y_binary_mask_tr[batch_index:(batch_size + batch_index), :, :, :]
    batch_index = batch_index + batch_size

    return X_image_batch, Y_label_batch, Y_edge_batch, Y_binary_mask_batch, batch_index


def Extract_Diff_Data(image, labels, edge, choose_index):
    X_image = []
    Y_label = []
    Y_edge = []
    for i in choose_index:
        X_image.append(image[i])
        Y_label.append(labels[i])
        Y_edge.append(edge[i])
    return np.array(X_image), np.array(Y_label), np.array(Y_edge)


def Aug_Train_Data(image, label, edge, binary_mask, batch_size, aug, IMAGE_SHAPE):
    """This function is used for performing data augmentation. 
    image: placeholder. shape: [Batch_Size, im_h, im_w, 3], tf.float32
    label: placeholder. shape: [Batch_Size, im_h, im_w, 1], tf.int64
    edge: placeholder. shape: [Batch_Size, im_h, im_w, 1], tf.int64
    binary_mask: placeholder. shape: [Batch_Size, im_h, im_w, 1], tf.int64
    
    Outputs:
    image: [Batch_Size, targ_h, targ_w, 3]
    label: [Batch_Size, targ_h, targ_w, 1]
    edge: [Batch_Size, targ_h, targ_w, 1]
    binary_mask: [Batch_Size, targ_h, targ_w, 1]
    
    """
    image = tf.cast(image, tf.int64)
    BigMatrix = tf.concat([image, label, edge, binary_mask], axis=3)
    target_height = IMAGE_SHAPE[0].astype('int32')
    target_width = IMAGE_SHAPE[1].astype('int32')
    if aug is True:
        BigMatrix_crop = tf.random_crop(BigMatrix, size=[batch_size, target_height, target_width, 6])
        k = tf.random_uniform(shape=[batch_size], minval=0, maxval=6.5, dtype=tf.float32)
        BigMatrix_rot = tf.contrib.image.rotate(BigMatrix_crop, angles=k)
        image_aug = tf.cast(BigMatrix_rot[:, :, :, 0:3], tf.float32)
        label_aug = BigMatrix_rot[:, :, :, 3]
        edge_aug = BigMatrix_rot[:, :, :, 4]
        binary_mask_aug = BigMatrix_rot[:, :, :, 5]
    else:
        BigMatrix_rot = tf.image.resize_image_with_crop_or_pad(BigMatrix, target_height, target_width)
        image_aug = tf.cast(tf.cast(BigMatrix_rot[:, :, :, 0:3], tf.uint8), tf.float32)
        label_aug = tf.cast(BigMatrix_rot[:, :, :, 3], tf.int64)
        edge_aug = tf.cast(BigMatrix_rot[:, :, :, 4], tf.int64)
        binary_mask_aug = tf.cast(BigMatrix_rot[:, :, :, 5], tf.int64)
    return image_aug, tf.expand_dims(label_aug, -1), tf.expand_dims(edge_aug, -1), tf.expand_dims(binary_mask_aug, -1)
